<?php

namespace Drupal\html_lang_override;

/**
 * @file
 * Contains \Drupal\html_lang_override\HtmlLangManager.
 */

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;

/**
 * Class HtmlLangManager provides various methods for language code overrides.
 *
 * @package Drupal\html_lang_override
 */
class HtmlLangManager {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Constructs a new HtmlLangManager object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  public function __construct(Connection $database, ConfigFactoryInterface $config_factory, LanguageManagerInterface $language_manager, TimeInterface $time) {
    $this->database = $database;
    $this->config = $config_factory->getEditable('html_lang_override.settings');
    $this->languageManager = $language_manager;
    $this->time = $time;
  }

  /**
   * Gets the default language code.
   *
   * @return string
   *   The default language code.
   */
  public function getDefaultLang() {
    if ($this->config->get('override_html_lang')) {
      return $this->config->get('default_lang');
    }
    else {
      return $this->languageManager->getDefaultLanguage()->getId();
    }
  }

  /**
   * Sets the default language code.
   *
   * @param string $nid
   *   Get language code for node.
   */
  public function getLangForNode($nid) {

    if (!$nid) {
      return NULL;
    }

    return $this->database->select('html_lang_override_node', 'hl')
      ->fields('hl', ['lang'])
      ->condition('nid', $nid)
      ->execute()
      ->fetchField() ?? NULL;
  }

  /**
   * Saves the language code for a node.
   *
   * @param int $nid
   *   The node ID.
   * @param string $lang
   *   The language code.
   */
  public function saveLangForNode($nid, $lang) {
    $this->database->merge('html_lang_override_node')
      ->key(['nid' => $nid])
      ->fields([
        'lang' => $lang,
        'changed' => $this->time->getRequestTime(),
        'created' => $this->time->getRequestTime(),
      ])
      ->execute();
  }

  /**
   * Gets the language code for a specific path.
   *
   * @param string $path
   *   The path to check.
   *
   * @return string
   *   The language code.
   */
  public function getLangForPath($path) {
    $all_path_overrides = (string) $this->config->get('route_path_override');
    if ($all_path_overrides) {
      $path_array = explode("\n", $all_path_overrides);
      foreach ($path_array as $item) {
        [$route, $language] = explode('|', trim($item));
        if (trim($path) == trim($route)) {
          return trim($language);
        }
      }
    }
    return NULL;
  }

  /**
   * Deletes the language code for a node.
   *
   * @param int $nid
   *   The node ID.
   */
  public function deleteLangForNode($nid) {
    $this->database->delete('html_lang_override_node')
      ->condition('nid', $nid)
      ->execute();
  }

}
